from ean_data import *
from ptn_data import *
from line_data import *
from od_data import *
from ip_model_tim_pass import *
from ip_model_tim_pass_cycle import *
from read_config import *
from read_csv import *
import sys

# Get Parameters from config file
# Config
config = ReadConfig(sys.argv[1])
# General instance data
directed = not config.get_bool_param('ptn_is_undirected')
period_length = config.get_int_param('period_length')
use_preprocessing = config.get_bool_param('tim_pass_use_preprocessing')
use_cycle_base_formulation = config.get_bool_param('tim_pass_use_cycle_base')
# Timetabling
global_min_wait_time = config.get_int_param('ean_default_minimal_waiting_time')
global_max_wait_time = config.get_int_param('ean_default_maximal_waiting_time')
global_min_trans_time = config.get_int_param('ean_default_minimal_change_time')
global_max_trans_time = config.get_int_param('ean_default_maximal_change_time')
# Timetabling - Objective
factor_travel_time = config.get_float_param('factor_travel_time')
factor_drive_time = config.get_float_param('factor_drive_time')
factor_transfer_time = config.get_float_param('factor_transfer_time')
factor_wait_time = config.get_float_param('factor_wait_time')
factor_penalty_time_slice = 1
factor_penalty_transfer = config.get_float_param('factor_penalty_transfer')
# Passenger routing
tim_pass_fix_passengers = config.get_bool_param('tim_pass_fix_passengers')
number_of_routed_od_pairs = config.get_int_param('number_of_routed_od_pairs')
# Solver (Gurobi)
time_limit = config.get_int_param('time_limit')
mip_gap = config.get_float_param('mip_gap')
# Filenames
stop_file_name = config.get_str_param('default_stops_file')
edge_file_name = config.get_str_param('default_edges_file')
line_concept_file_name = config.get_str_param('default_lines_file')
od_file_name = config.get_str_param('default_od_file')
activity_file_name = config.get_str_param('default_activities_periodic_file')
event_file_name = config.get_str_param('default_events_periodic_file')
timetable_file_name = config.get_str_param('default_timetable_periodic_file')

# Read PTN
ptn = Ptn()
read_ptn(stop_file_name, edge_file_name, ptn, directed, global_min_wait_time, global_max_wait_time,
         global_min_trans_time, global_max_trans_time)

# Read OD-matrix
od = OD()
read_od_matrix(od_file_name, ptn, od, global_n_time_slices=1, global_min_turn_around_time=-1,
               period_length=period_length)

# Activating the largest OD-pairs
for index, od_pair in enumerate(sorted(od.get_all_od_pairs(), key=lambda x: x.get_total_passengers(), reverse=True)):
    if index < number_of_routed_od_pairs:
        od_pair.active = True

# Read line pool
line_pool = LinePool()
read_line_pool(line_concept_file_name, line_pool, ptn, directed, read_line_concept=True, restrict_to_frequency_1=False)

# Construct EAN
ean = Ean(ptn, line_pool, od, multiple_frequencies=True, period_length=period_length)

# Solve integrated timetabling and passenger routing problem
if use_cycle_base_formulation:
    solve_cycle_base(ean, ptn, line_pool, od, period_length, activity_file_name, event_file_name, timetable_file_name,
                     factor_travel_time, factor_drive_time, factor_transfer_time, factor_wait_time,
                     factor_penalty_transfer, factor_penalty_time_slice, tim_pass_fix_passengers, use_preprocessing,
                     time_limit, mip_gap, write_lp_output=False)
else:
    solve(ean, ptn, line_pool, od, period_length, activity_file_name, event_file_name, timetable_file_name,
          factor_travel_time, factor_drive_time, factor_transfer_time, factor_wait_time, factor_penalty_transfer,
          factor_penalty_time_slice, tim_pass_fix_passengers, use_preprocessing, time_limit, mip_gap,
          write_lp_output=False)
